// Process.c is the application layer.  All function calls are hardware ambivalent.

#include <stdint.h>              // Find integer types like "uint8_t"  
#include "Eve2_81x.h"            // Matrix Orbital Eve2 Driver
#include "Arduino_AL.h"          // include the hardware specific abstraction layer header for the specific hardware in use.
#include "process.h"             // Every c file has it's header and this is the one for this file

void SelectScreen(uint8_t ID)
{
  switch(ID)
  {   
  case SCR_MO:
    MakeScreen_MO_EVE();
    break;  
  case SCR_BLK:
    MakeScreen_Black();            // Button and text screen
    break; 
  case SCR_WHT:
    MakeScreen_White();            // Button and text screen
    break; 
  case SCR_RED:
    MakeScreen_Red();            // Button and text screen
    break; 
  case SCR_GRE:
    MakeScreen_Green();            // Button and text screen
    break; 
  case SCR_BLU:
    MakeScreen_Blue();            // Button and text screen
    break;     
  case SCR_BOX:
    MakeScreen_Box();            // Button and text screen
    break;     
  case SCR_JPG: 
    MakeScreen_JPEG("bob.jpg", 208, 86, 0, 0, 0);                          
    break;  
  case SCR_RAW:
    MakeScreen_Bitmap_DL("bob.raw", 208, 86, 0,0,0);
    break;          
  default:
    break;
  }
}


// Sound demo - written to the Co-processor ( AKA GPU and FIFO )
void MakeScreen_MO_EVE(void)
{
//  Log("Enter Button\n");

  Send_CMD(CMD_DLSTART);
  Cmd_Gradient(0, 0, 0xA1E1FF, 0, 250, 0x000080);           // The whole screen is a color gradient in blue
  
  Cmd_Text(DWIDTH/2, 20, 30, OPT_CENTERX, "Matrix Orbital");  // White text
  Cmd_Text(DWIDTH/2, 50, 30, OPT_CENTERX, "EVE Display Series");  // White text
  
  Send_CMD(DISPLAY());
  Send_CMD(CMD_SWAP);  
  UpdateFIFO();                                       // Trigger the CoProcessor to start processing the FIFO
}

void MakeScreen_Black(void)
{
  Send_CMD(CMD_DLSTART);
  Send_CMD(CLEAR_COLOR_RGB(0,0,0));
  Send_CMD(CLEAR(1,1,1));
  Send_CMD(DISPLAY());
  Send_CMD(CMD_SWAP);
  UpdateFIFO();                                               // Trigger the CoProcessor to start processing commands out of the FIFO   
}

void MakeScreen_White(void)
{
  Send_CMD(CMD_DLSTART);
  Send_CMD(CLEAR_COLOR_RGB(255,255,255));
  Send_CMD(CLEAR(1,1,1));
  Send_CMD(DISPLAY());
  Send_CMD(CMD_SWAP);
  UpdateFIFO();                                               // Trigger the CoProcessor to start processing commands out of the FIFO   
}

void MakeScreen_Red(void)
{
  Send_CMD(CMD_DLSTART);
  Send_CMD(CLEAR_COLOR_RGB(255,0,0));
  Send_CMD(CLEAR(1,1,1));
  Send_CMD(DISPLAY());
  Send_CMD(CMD_SWAP);
  UpdateFIFO();                                               // Trigger the CoProcessor to start processing commands out of the FIFO   
}

void MakeScreen_Blue(void)
{
  Send_CMD(CMD_DLSTART);
  Send_CMD(CLEAR_COLOR_RGB(0,255,0));
  Send_CMD(CLEAR(1,1,1));
  Send_CMD(DISPLAY());
  Send_CMD(CMD_SWAP);
  UpdateFIFO();                                               // Trigger the CoProcessor to start processing commands out of the FIFO   
}

void MakeScreen_Green(void)
{
  Send_CMD(CMD_DLSTART);
  Send_CMD(CLEAR_COLOR_RGB(0,0,255));
  Send_CMD(CLEAR(1,1,1));
  Send_CMD(DISPLAY());
  Send_CMD(CMD_SWAP);
  UpdateFIFO();                                               // Trigger the CoProcessor to start processing commands out of the FIFO   
}

void MakeScreen_Box(void)
{
  Send_CMD(CMD_DLSTART);
  Send_CMD(CLEAR_COLOR_RGB(0,0,0));
  Send_CMD(CLEAR(1,1,1));
  
  Send_CMD(COLOR_RGB(64, 64, 64));                          // Change color of following text to grey
  Cmd_FGcolor(0xffffff);                                    // Change button face color to a mild green  
  Cmd_Button(56, 8, 208, 86, 28, 0, " ");  
  
  Send_CMD(DISPLAY());
  Send_CMD(CMD_SWAP);  
  UpdateFIFO();                                            // Trigger the CoProcessor to start processing the FIFO
}


// Using raw uncompressed bitmap data, the CoProcessor is not needed for inflation and the data is 
// stored into RAM_G without the use of the FIFO / CoProcessor.
// This function uses only the Display List and not FIFO
void MakeScreen_Bitmap_DL(uint8_t *filename, uint16_t Xsize, uint16_t Ysize, uint16_t Xloc, uint16_t Yloc, uint8_t Reference)
{
  uint32_t BMPBaseAdd = RAM_G + 0x40000UL;                               // Hard-coded Address lacks panache
  
  Load_RAW(BMPBaseAdd, filename);                                        // Load a bitmap into RAM_G
  Log("after loadRAW \n"); 
  
  // Screen start
  wr32(RAM_DL + 0, CLEAR_COLOR_RGB(200,200,200));                        // Set the color for clearing to whitish
  wr32(RAM_DL + 4, CLEAR(1, 1, 1));                                      // clear screen
  
  // Define the bitmap
  wr32(RAM_DL + 8, BITMAP_HANDLE(Reference));                            // handle for this bitmap
  wr32(RAM_DL + 12, BITMAP_SOURCE(BMPBaseAdd));                          // Starting address in GRAM
  wr32(RAM_DL + 16, BITMAP_LAYOUT(L1, 26, Ysize));              // format, stride, height 
  wr32(RAM_DL + 20, BITMAP_TRANSFORM_A(167)); //167 = 256/(302/208)             
  wr32(RAM_DL + 24, BITMAP_TRANSFORM_E(215));//215 = 256/(102/867)
  wr32(RAM_DL + 28, BITMAP_SIZE(NEAREST, BORDER, BORDER, 320, 102)); // define the holding area for the bitmap
  
  // Place the bitmap
  wr32(RAM_DL + 32, BEGIN(BITMAPS));
  wr32(RAM_DL + 36, BITMAP_HANDLE(Reference));
  wr32(RAM_DL + 40, VERTEX2II(Xloc, Yloc, Reference, 0));                // Define the placement position of the previously defined holding area.                                                                          
  wr32(RAM_DL + 44, END());                                              // end placing bitmaps

  wr32(RAM_DL + 48, DISPLAY());                                          // End display list
  wr8(REG_DLSWAP + RAM_REG, DLSWAP_FRAME);                               // swap display lists

  Log("Leave Bitmap DL\n");
}

// Decompress a JPEG compressed image from SD card into RAM_G
void MakeScreen_JPEG(uint8_t *filename, uint16_t Xsize, uint16_t Ysize, uint16_t Xloc, uint16_t Yloc, uint8_t Reference)
{
  uint32_t tmp;
  uint32_t BMPBaseAdd = RAM_G + 0x80000UL;                           // Hard-coded Address lacks panache

  tmp = Load_JPG(BMPBaseAdd, 0, filename);                           // Load a bitmap into RAM_G
  Log("JPG used 0x%08lx\n", tmp - BMPBaseAdd); 
  
  // Screen start
  Send_CMD(CMD_DLSTART);                                             // Start a new display list
  Send_CMD(CLEAR_COLOR_RGB(255,255,255));                            // Set the color for clearing to full white
  Send_CMD(CLEAR(1,1,1));                                            // clear screen 
  
  // Define the bitmap
  Send_CMD(BITMAP_HANDLE(Reference));                                // handle for this bitmap
  Send_CMD(BITMAP_SOURCE(BMPBaseAdd));                          // Starting address in GRAM  
  Send_CMD(BITMAP_LAYOUT(RGB565, Xsize*2, Ysize));                           // format, stride, height   
  Send_CMD(BITMAP_TRANSFORM_A(167));//167 = 256/(302/208)
  Send_CMD(BITMAP_TRANSFORM_E(215));//215 = 256/(102/867)
  Send_CMD(BITMAP_SIZE(NEAREST, BORDER, BORDER, 320, 102));
  
  
  // Place the bitmap
  Send_CMD(BEGIN(BITMAPS));
  Send_CMD(BITMAP_HANDLE(Reference));
  Send_CMD(VERTEX2II(Xloc, Yloc, Reference, 0));                     // Define the placement position of the previously defined holding area.
  Send_CMD(END());                                                   // end placing bitmaps
  
  Send_CMD(DISPLAY());                                               // End display list
  Send_CMD(CMD_SWAP);                                                // Activate this display list
  UpdateFIFO();                                                      // Trigger the CoProcessor to start processing the FIFO

  Log("Leave jpeg\n");
}

// This define is for the size of the buffer we are going to use for data transfers.  It is 
// sitting here so uncomfortably because it is a silly tiny buffer in Arduino Uno and you
// will want a bigger one if you can get it.  Redefine this and add a nice buffer to Load_ZLIB()
#define COPYBUFSIZE WorkBuffSz

// Load a compressed bitmap from SD card into RAM_G at address "BaseAdd"
// The file will have been processed by "img_cvt.exe" from FTDI
// Return value is the last RAM_G address used during the inflation operation - you might want it
uint32_t Load_ZLIB(uint32_t BaseAdd, char *filename) 
{
  uint32_t Remaining;
  uint16_t ReadBlockSize = 0;

  // Open the file on SD card by name
  FileOpen(filename, FILEREAD);
  if(!myFileIsOpen())
  {
    Log("%s not open\n", filename);
    FileClose();
    return false;
  }
  Log("Done");
  
  Remaining = FileSize();                                    // Store the size of the currently opened file
  
  Send_CMD(CMD_INFLATE);                                     // Tell the CoProcessor to prepare for compressed data
  Send_CMD(BaseAdd);                                         // This is the address where decompressed data will go 

  while (Remaining)
  {
    if (Remaining > COPYBUFSIZE)
      ReadBlockSize = COPYBUFSIZE;
    else
      ReadBlockSize = Remaining;
    
    FileReadBuf(LogBuf, ReadBlockSize);                      // Read a block of data from the file
    CoProWrCmdBuf(LogBuf, ReadBlockSize);                    // write the block to FIFO - Does FIFO triggering
    Remaining -= ReadBlockSize;                              // Reduce remaining data value by amount just read
  }
  FileClose();

  Wait4CoProFIFOEmpty();                                     // wait here until the coprocessor has read and executed every pending command.

  // Get the address of the last RAM location used during inflation
  Cmd_GetPtr();                                              // FifoWriteLocation is updated twice so the data is returned to it's updated location - 4
  UpdateFIFO();                                              // force run the GetPtr command
  return (rd32(FifoWriteLocation + RAM_CMD - 4));            // The result is stored at the FifoWriteLocation - 4 (because FTDI is Chaotic Evil)
}         

// Load a JPEG image from SD card into RAM_G at address "BaseAdd"
// Return value is the last RAM_G address used during the jpeg decompression operation.
// The function is virtually identical to Load_ZLIB as the only difference is the memory operation
uint32_t Load_JPG(uint32_t BaseAdd, uint32_t Options, char *filename) 
{
  uint32_t Remaining;
  uint16_t ReadBlockSize = 0;

  // Open the file on SD card by name
  FileOpen(filename, FILEREAD);
  if(!myFileIsOpen())
  {
    Log("%s not open\n", filename);
    FileClose();
    return false;
  }
  
  Remaining = FileSize();                                    // Store the size of the currently opened file
  
  Send_CMD(CMD_LOADIMAGE);                                   // Tell the CoProcessor to prepare for compressed data
  Send_CMD(BaseAdd);                                         // This is the address where decompressed data will go 
  Send_CMD(Options);                                         // Send options (options are mostly not obviously useful)

  while (Remaining)
  {
    if (Remaining > COPYBUFSIZE)
      ReadBlockSize = COPYBUFSIZE;
    else
      ReadBlockSize = Remaining;
    
    FileReadBuf(LogBuf, ReadBlockSize);                      // Read a block of data from the file
    CoProWrCmdBuf(LogBuf, ReadBlockSize);                    // write the block to FIFO - Does FIFO triggering
    Remaining -= ReadBlockSize;                              // Reduce remaining data value by amount just read
  }
  FileClose();

  Wait4CoProFIFOEmpty();                                     // wait here until the coprocessor has read and executed every pending command.

  // Get the address of the last RAM location used during inflation
  Cmd_GetPtr();                                              // FifoWriteLocation is updated twice so the data is returned to it's updated location - 4
  UpdateFIFO();                                              // force run the GetPtr command
  return (rd32(FifoWriteLocation + RAM_CMD - 4));            // The result is stored at the FifoWriteLocation - 4 (Yes, this is unexpected and random)
}         

// Load a raw (image data) bitmap from SD card into RAM_G at address 0
// The file will have been processed by "img_cvt.exe" from FTDI
// While the memory usage is not a mystery, the next available address is returned for convenience.
uint32_t Load_RAW(uint32_t BaseAdd, char *filename)                                 
{
  uint32_t Remaining;
  uint16_t ReadBlockSize = 0;
  uint32_t Add_GRAM = BaseAdd;

  // Open the file on SD card by name
  FileOpen(filename, FILEREAD);
  if(!myFileIsOpen())
  {
    Log("%s not open\n", filename);
    FileClose();
    return false;
  }
  
  Remaining = FileSize();                                      // Store the size of the currently opened file

  while (Remaining)
  {
    if (Remaining > COPYBUFSIZE)
      ReadBlockSize = COPYBUFSIZE;
    else
      ReadBlockSize = Remaining;
    
    FileReadBuf(LogBuf, ReadBlockSize);                        // Read a block of data from the file
    Add_GRAM = WriteBlockRAM(Add_GRAM, LogBuf, ReadBlockSize); // write the block to RAM_G
    Remaining -= ReadBlockSize;                                // Reduce remaining data value by amount just read
  }
  FileClose();
  return (Add_GRAM);
}

// Is there a physical key press?
uint8_t CheckKeys(void)
{
  if(!ReadPin(Button1_PIN))
    return(1);
  if(!ReadPin(Button2_PIN))
    return(2);
  if(!ReadPin(Button3_PIN))
    return(3);

  return(0);
}
